;
$DATE (06.12.2010)
$TITLE (TestSerial.a51)
$PAGELENGTH(56)
$PAGEWIDTH(150)
$DEBUG
$XREF
$NOLIST
$NOMOD51
$INCLUDE(89S8252.MCU)
$LIST
;

;********************************************************************************************************
; Lesen und Schreiben von Bytes ber die serielle Schnittstelle (direkt)
; J. Rathlev, Dez. 2003
;********************************************************************************************************

      NAME      TestSerial

;********************************************************************************************************
; Konstanten
; ==========

CODEMEM EQU     8000H

LED     EQU     P1.7    ; Portanschluss fr LED

LF      EQU     0AH
CR      EQU     0DH

;********************************************************************************************************
; RAM-Daten
; =========

        DSEG    AT      30H

;********************************************************************************************************
; Programmstart

        CSEG    AT      CODEMEM

        LJMP    Start


;********************************************************************************************************
; Baudrateneinstellungen:
; SMOD = 1 ==> TH1 = 256 - (2 * f/Hz) / (384 * BR)

; 22,1184 MHz -   2400 : 208     24 MHz - 2400 : 204        12 MHz - 2400 : 230
;             -   4800 : 232            - 4800 : 230               - 4800 : 243
;             -   9600 : 244            - 9600 : 243
;             -  19200 : 250
;             -  38400 : 253
;             -  57600 : 254
;             - 115200 : 255

;********************************************************************************************************
; RS232 initalisieren

        ORG     CODEMEM+30H

Init:   MOV     A,#244          ; BR-Count (s.o.)
        MOV     TH1,A           ; load value

        MOV     A,TMOD
        ANL     A,#0FH
        ADD     A,#20H
        MOV     TMOD,A          ; timer 1 mode 2 (8 bit, auto-reload)
        CLR     SM0             ; serial mode 1 (8-bit)
        SETB    SM1
        SETB    TR1             ; start timer
        MOV     A,PCON
        ORL     A,#80H
        MOV     PCON,A          ; SMOD=1 (Baudrate x 2)
        SETB    REN             ; Enable receiver
        SETB    TI              ; set TI for transmit buffer empty
        CLR     RI              ; clear SI for receive interrupt
        RET

;********************************************************************************************************
; Ein Zeichen ber interne ser. Schnittstelle ausgeben
; Zeichen in A

WRITECHAR:
WWAIT:  JNB     TI,WWAIT        ; warten
        MOV     SBUF,A          ; Ausgabe
        CLR     TI
WRET:   RET



;********************************************************************************************************
; Nullterm. String in Daten-Speicher ber interne ser. Schnittstelle ausgeben 
; und Cr-Lf anhngen
; Zeiger auf String in DPTR

WRITESTRING:
        MOVX    A,@DPTR
        INC     DPTR
        JZ      WSEND
        ACALL   WRITECHAR
        SJMP    WRITESTRING
        
WSEND:  MOV     A,#CR
        ACALL   WRITECHAR
        MOV     A,#LF
        ACALL   WRITECHAR
        RET

;********************************************************************************************************
; Nulltem. String aus Code-Mem. ber interne ser. Schnittstelle ausgeben 
; Zeiger auf String in DPTR

WRITEFIXSTR:
        CLR     A 
        MOVC    A,@A+DPTR
        INC     DPTR
        JZ      WFEND
        ACALL   WRITECHAR
        SJMP    WRITEFIXSTR
        
WFEND:  RET

;********************************************************************************************************
; Hauptprogramm

Start:  MOV     SP,#STACKSTART
        CALL    INIT
        MOV     DPTR,#TEXT
        CALL    WRITEFIXSTR
LOOP:   JNB     RI,LOOP         ; warten
        CLR     RI
        MOV     A,SBUF          ; Zeichen lesen
        CPL     LED
        CALL    WRITECHAR       ; Echo
        SJMP    LOOP



TEXT:   DB      'Hallo, hier ist der Mikrocontroller!',13,10,'Tippen Sie ein Zeichen: ',0

;********************************************************************************************************
        DSEG

STACKSTART:
        DS      1       ; stack: rest of data memory

END
